@@ -39,6 +39,8 @@ class R3sponse(object):
 		variables={}, 
 		# log log level of the message (int).
 		log_level=None, 
+		# the required log level for when printed to console (leave None to use self.log_level).
+		required_log_level=None, 
 		# save the error to the logs file.
 		save=False,
 		# return as a django JsonResponse.
@@ -51,7 +53,7 @@ class R3sponse(object):
 		})
 		for key, value in variables.items():
 			response[key] = value
-		r3sponse.log(message=response["message"], log_level=log_level, save=save)
+		self.log(message=response["message"], log_level=log_level, save=save, required_log_level=required_log_level)
 		if django: 
 			try:
 				response = JsonResponse(response.dict(), safe=False)
@@ -63,6 +65,8 @@ class R3sponse(object):
 		error="", 
 		# log log level of the message (int).
 		log_level=None, 
+		# the required log level for when printed to console (leave None to use self.log_level).
+		required_log_level=None, 
 		# save the error to the erros file.
 		save=False,
 		# return as a django JsonResponse.
@@ -75,7 +79,7 @@ class R3sponse(object):
 			"message":None,
 			"error":error,
 		})
-		r3sponse.log(error=response["error"], log_level=log_level, save=save)
+		self.log(error=response["error"], log_level=log_level, save=save, required_log_level=required_log_level)
 		if traceback:
 			raise ValueError(response["error"])
 		if django: 
@@ -87,7 +91,7 @@ class R3sponse(object):
 		#
 
 	# parameter functions.
-	def get_request_parameter(self, request, identifier):
+	def get_request_parameter(self, request, identifier, traceback=None):
 		response = self.default_response()
 		format = None
 		if ":" in identifier:
@@ -100,7 +104,11 @@ class R3sponse(object):
 		else:
 			variable = request.GET.get(identifier)
 		if variable in ["", None]:
-			return variable, self.error(f"Define parameter: [{identifier}].")
+			if traceback != None:
+				return variable, self.error(f"{traceback}: Define parameter: [{identifier}].")
+			else:
+				return variable, self.error(f"Define parameter: [{identifier}].")
+
 		elif format != None:
 			if format in ["str", "string"]: variable = str(variable)
 			elif format in ["int", "integer"]: variable = int(variable)
@@ -115,12 +123,12 @@ class R3sponse(object):
 			"key":identifier,
 			"value":variable,
 		})
-	def get_request_parameters(self, request, identifiers=[], optional=False, empty_value=None):
+	def get_request_parameters(self, request, identifiers=[], optional=False, empty_value=None, traceback=None):
 		if isinstance(identifiers, str):
 			return self.error("__get_request_params__ is used to retrieve several identifiers (array format not string).")
 		params = self.ResponseObject()
 		for param in identifiers:
-			param_value, response = self.get_request_parameter(request, param)
+			param_value, response = self.get_request_parameter(request, param, traceback=traceback)
 			param = param.split(":")[0]
 			if response["error"] != None: 
 				if optional:
@@ -136,10 +144,13 @@ class R3sponse(object):
 		return params, self.success(f"Succesfully retrieved {len(params)} request parameter(s).")
 
 	# check parameters.
-	def check_parameter(self, parameter=None, name="parameter", empty_value=None):
+	def check_parameter(self, parameter=None, name="parameter", empty_value=None, traceback=None):
 		response = self.default_response()
 		if parameter == empty_value: 
-			return self.error(f"Define parameter [{name}].")
+			if traceback != None:
+				return self.error(f"{traceback}: Define parameter [{name}].")
+			else:
+				return self.error(f"Define parameter [{name}].")
 		if ":" in name:
 			name,formats = name.split(":")
 			while True:
@@ -150,10 +161,10 @@ class R3sponse(object):
 			if param_format not in formats:
 				return self.error(f"Incorrect parameter [{name}:{param_format}] format, correct format(s): {Array(path=False, array=formats).string(joiner=', ')}.")
 		return self.success(f"Succesfully checked parameter [{name}].")
-	def check_parameters(self, parameters={"parameter":None}, empty_value=None):
+	def check_parameters(self, parameters={"parameter":None}, empty_value=None, traceback=None):
 		response = self.default_response()
 		for id, value in parameters.items():
-			response = self.check_parameter(value, id, empty_value=empty_value)
+			response = self.check_parameter(value, id, empty_value=empty_value, traceback=traceback)
 			if response["error"] != None: return response
 		return response
 
@@ -173,6 +184,8 @@ class R3sponse(object):
 		# optionals:
 		# the log level for printing to console.
 		log_level=0,
+		# the required log level for when printed to console (leave None to use self.log_level).
+		required_log_level=None, 
 		# save to log file.
 		save=False,
 		# save errors only (for option 2 only).
@@ -191,8 +204,13 @@ class R3sponse(object):
 			msg = f"Error: {error}"
 		else: 
 			msg = message
+		if required_log_level == None: required_log_level = self.log_level
 		try:
-			comparison = log_level != None and log_level >= self.log_level
+			required_log_level = int(required_log_level)
+		except: 
+			required_log_level = 0
+		try:
+			comparison = log_level != None and log_level >= required_log_level
 		except TypeError as e:
 			if "not supported between instances of 'dict' and 'int'" in f"{e}":
 				raise TypeError(f"You most likely returned a r3sponse.error_response when you meant a r3sponse.success_response, error: {e}")
@@ -397,7 +415,7 @@ class ResponseObject(object):
 	# crash the error message.
 	def crash(self, error="ValueError", traceback=True, json=False):
 		if json:
-			r3sponse.log(error=self["error"], json=json)
+			self.log(error=self["error"], json=json)
 			sys.exit(1)
 		else:
 			#if not traceback:
@@ -504,7 +522,7 @@ class ResponseObject(object):
 	# dump json string.
 	def json(self, sorted=False, reversed=False, indent=4, dictionary=None, ):
 		if dictionary == None: dictionary = self.dict()
-		return json.dumps(self.serialize(json=False, sorted=sorted, reversed=reversed, dictionary=dictionary), indent=indent)
+		return json.dumps(self.serialize(json=False, sorted=sorted, reversed=reversed, dictionary=dictionary), indent=indent).replace(': "False"', ': false').replace(': "True"', ': true').replace(': "None"', ': null')
 	# serialize dict.
 	def serialize(self, sorted=False, reversed=False, json=False, dictionary=None):
 		if dictionary == None: dictionary = self.dict()
